import { Creator } from "../typings/Creator";
/**
 * Utility functions for ORM Entities.
 *
 * @author Jeongho Nam - https://github.com/samchon
 */
export declare namespace EntityUtil {
    interface ITableInfo {
        target: Creator<object>;
        schema: string;
        name: string;
        primary: string;
        uniques: string[][];
        children: ITableInfo.IChild[];
    }
    namespace ITableInfo {
        interface IChild {
            info: ITableInfo;
            foreign: string;
        }
    }
    /**
     * Get table name.
     *
     * @param entity Target entity type
     * @returns Table name
     */
    function name<Entity extends object>(entity: Creator<Entity>): string;
    /**
     * Get table info.
     *
     * @param entity Target entity type
     * @returns Table info
     */
    function info<Entity extends object>(entity: Creator<Entity>): ITableInfo;
    /**
     * Merge duplicate records into one.
     *
     * Absords and merges multiple entity records (*duplicates*) into one (*original*).
     * That is, all records listed in *duplicates* would be erased, and instead,
     * references to entities of all records subordinate to duplicate records would be
     * replaced with original ones.
     *
     * During unification, if there're some children entities dependent on `Entity` and
     * their foreign columns referencing `Entity` are belonged to unique constraint,
     * they would be unified recursively.
     *
     * @template Entity Target entity type
     * @param original Original record to absorb
     * @param duplicates Duplicated records to be absorbed
     */
    function unify<Entity extends object>(original: Entity, duplicates: Entity[]): Promise<void>;
}
