"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Paginator = void 0;
var tstl_1 = require("tstl");
/**
 * Pagination utility.
 *
 * @author Jeongho Nam - https://github.com/samchon
 */
var Paginator;
(function (Paginator) {
    /**
     * Pagination from regular ORM entity instances with transformation.
     *
     * Paginate regular ORM entity instances by getting from
     * {@link SelectQueryBuilder.getMany} method and transforming them to *Output* type.
     *
     * If you want to paginate not regular data, but raw data getting by
     * {@link SelectQueryBuilder.getRawMany}, use {@link rough} or {@link raw}
     * function instead. For referecen, different between {@link rough} and {@link raw}
     * is whether do transform or not.
     *
     * @template Entity Target entity type
     * @param stmt Select query builder for Entity type
     * @returns Currying function
     */
    Paginator.regular = function (stmt) {
        /**
         * @template Output Output data type
         * @param transformer Transformer function coverting Entity to Output
         * @returns Currying function
         */
        return function (transformer) {
            /**
             * @param request Page request info
             * @returns Paginated Output data
             */
            return function (request) {
                return _Paginate(stmt, function (stmt) { return stmt.getMany(); }, request, transformer);
            };
        };
    };
    /**
     * Pagination from raw data with transformation.
     *
     * Paginate raw data by getting from {@link SelectQueryBuilder.getRawMany}
     * method and transforming them to *Output* type.
     *
     * When you want to paginate regular ORM entity instances, use {@link regular}
     * function instead. Otherwise, you do not need transformation, just use
     * {@link raw} function.
     *
     * @template Entity Target entity type
     * @param stmt Select query builder for Entity type
     * @returns Currying function
     */
    Paginator.rough = function (stmt) {
        /**
         * @template Raw Raw data type
         * @template Output Output data type
         * @param transformer Transformer function coverting Entity to Output
         * @returns Currying function
         */
        return function (transformer) {
            /**
             * @param request Page request info
             * @returns Paginated Output data
             */
            return function (request) {
                return _Paginate(stmt, function (stmt) { return stmt.getRawMany(); }, request, transformer);
            };
        };
    };
    /**
     * Pagination from raw data without transformation.
     *
     * Paginate raw data by getting from {@link SelectQueryBuilder.getRawMany}
     * method. Unlink {@link rough}, this function does not transform raw data.
     * Therefore, the *Output* type must be constructed only by raw query.
     *
     * Otherwise you want to transform raw data, use {@link rough} function
     * instead. On contrary, if you want to paginate regular ORM entity instances,
     * use {@link regular} function instead.
     *
     * @template Entity Target entity type
     * @param stmt Select query builder for Entity type
     * @returns Currying function
     */
    Paginator.raw = function (stmt) {
        /**
         * @template Output Output data type
         * @param request Page request info
         * @returns Paginated Output data
         */
        return function (request) {
            return _Paginate(stmt, function (stmt) { return stmt.getRawMany(); }, request, undefined);
        };
    };
    /**
     * Add order by statements to query builder.
     *
     * @param stmt Target select query builder
     * @param fieldList List of fields to order with direction
     * @param dictionary Dictionary for real column names if required
     * @param groupped Whether be groupped or not
     * @return Query builder itself
     */
    function orderBy(stmt, fieldList, dictionary, groupped) {
        // SPECIALIZATION
        fieldList.forEach(function (symbol, index) {
            var direction = symbol[0] !== "-" ? "ASC" : "DESC";
            var field = symbol.substring(1);
            var key = (function () {
                if (dictionary) {
                    var found = dictionary[field];
                    if (found)
                        return typeof found === "string" ? found : found();
                }
                return field;
            })();
            // DO ORDER
            var target = groupped === true ? "MAX(".concat(key, ")") : key;
            if (index === 0)
                stmt.orderBy(target, direction);
            else
                stmt.addOrderBy(target, direction);
        });
        return stmt;
    }
    Paginator.orderBy = orderBy;
    /**
     * Follow order of source.
     *
     * Sort target array by following the order of source array.
     *
     * @param target Target array to sort
     * @param source Source array to be referenced
     * @param keyGetter Key getter function from any record
     * @return Target array itself
     */
    function follow(target, source, keyGetter) {
        var e_1, _a;
        if (keyGetter === void 0) { keyGetter = function (elem) { return elem.id; }; }
        var dict = new tstl_1.HashMap();
        try {
            for (var target_1 = __values(target), target_1_1 = target_1.next(); !target_1_1.done; target_1_1 = target_1.next()) {
                var o = target_1_1.value;
                dict.emplace(keyGetter(o), o);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (target_1_1 && !target_1_1.done && (_a = target_1.return)) _a.call(target_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        source.forEach(function (item, index) {
            target[index] = dict.get(item.id);
        });
        return target;
    }
    Paginator.follow = follow;
    function _Paginate(stmt, accessor, request, postProcess) {
        return __awaiter(this, void 0, void 0, function () {
            var total_count, total_pages, index, raw, data, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        // NORMALIZE INPUT
                        request.limit =
                            request.limit !== undefined ? Number(request.limit) : Paginator.DEFAULT_LIMIT;
                        return [4 /*yield*/, stmt.getCount()];
                    case 1:
                        total_count = _b.sent();
                        total_pages = request.limit !== 0 ? Math.ceil(total_count / request.limit) : 0;
                        request.page =
                            request.page !== undefined
                                ? Math.max(1, Math.min(Number(request.page), total_pages))
                                : 1;
                        index = stmt
                            .clone()
                            .offset(request.limit * Math.max(request.page - 1, 0))
                            .limit(request.limit);
                        return [4 /*yield*/, accessor(index)];
                    case 2:
                        raw = _b.sent();
                        if (!(postProcess && raw.length !== 0)) return [3 /*break*/, 4];
                        return [4 /*yield*/, postProcess(raw)];
                    case 3:
                        _a = _b.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        _a = raw;
                        _b.label = 5;
                    case 5:
                        data = _a;
                        // RETURNS
                        return [2 /*return*/, {
                                pagination: {
                                    page: request.page,
                                    limit: request.limit,
                                    total_count: total_count,
                                    total_pages: total_pages,
                                },
                                data: data,
                            }];
                }
            });
        });
    }
    Paginator.DEFAULT_LIMIT = 100;
})(Paginator = exports.Paginator || (exports.Paginator = {}));
//# sourceMappingURL=Paginator.js.map